@@ -2,16 +2,6 @@
 
 import numpy as np
 
-def generateIncludes(chunks: list, log) -> np.ndarray:
-    arrayLen = chunks[-1][1]
-    includeFrame = np.zeros((arrayLen), dtype=np.bool_)
-
-    for item in chunks:
-        if(item[2] == 1):
-            includeFrame[item[0]:item[1]] = True
-    return includeFrame
-
-
 def combineArrs(audioList: np.ndarray, motionList: np.ndarray, based: str,
     log) -> np.ndarray:
 
@@ -97,7 +87,7 @@ def audioToHasLoud(audioData: np.ndarray, sampleRate: int, silentT: float,
 
 # Motion detection algorithm based on this blog post:
 # https://pyimagesearch.com/2015/05/25/basic-motion-detection-and-tracking-with-python-and-opencv/
-def motionDetection(path: str, ffprobe: str, motionThreshold: float, log,
+def motionDetection(path: str, ffprobe, motionThreshold: float, log,
     width: int, dilates: int, blur: int) -> np.ndarray:
 
     import cv2
@@ -109,11 +99,11 @@ def motionDetection(path: str, ffprobe: str, motionThreshold: float, log,
     # Find total frames
     if(path.endswith('.mp4') or path.endswith('.mov')):
         # Query Container
-        cmd = [ffprobe, '-v', 'error', '-select_streams', 'v:0', '-show_entries',
+        cmd = [ffprobe.getPath(), '-v', 'error', '-select_streams', 'v:0', '-show_entries',
             'stream=nb_frames', '-of', 'default=nokey=1:noprint_wrappers=1', path]
     else:
         # Count the number of frames (slow)
-        cmd = [ffprobe, '-v', 'error', '-count_frames', '-select_streams', 'v:0',
+        cmd = [ffprobe.getPath(), '-v', 'error', '-count_frames', '-select_streams', 'v:0',
             '-show_entries', 'stream=nb_read_frames', '-of',
             'default=nokey=1:noprint_wrappers=1', path]
 
@@ -207,11 +197,16 @@ def removeSmall(hasLoud: np.ndarray, lim: int, replace: bool, with_: bool) -> np
     return hasLoud
 
 
-def setRange(includeFrame: np.ndarray, syntaxRange, fps: float, with_: bool) -> np.ndarray:
+def setRange(includeFrame: np.ndarray, syntaxRange, fps: float, with_: bool, log) -> np.ndarray:
     end = len(includeFrame) - 1
     for item in syntaxRange:
         pair = []
 
+        if(item.count('-') > 1):
+            log.error('Too many deliminators!')
+        if(item.count('-') < 1):
+            log.error('Invalid range. Use range syntax. ex: 5-10')
+
         for num in item.split('-'):
             if(num == 'start'):
                 pair.append(0)
@@ -266,11 +261,11 @@ def applySpacingRules(hasLoud: np.ndarray, fps: float, frameMargin: int,
 
     # Apply ignore rules if applicable.
     if(ignore != []):
-        includeFrame = setRange(includeFrame, ignore, fps, True)
+        includeFrame = setRange(includeFrame, ignore, fps, True, log)
 
     # Cut out ranges.
     if(cutOut != []):
-        includeFrame = setRange(includeFrame, cutOut, fps, False)
+        includeFrame = setRange(includeFrame, cutOut, fps, False, log)
 
     # Remove small clips/cuts created by applying other rules.
     includeFrame = cook(includeFrame, minClip, minCut)
@@ -284,6 +279,4 @@ def applySpacingRules(hasLoud: np.ndarray, fps: float, frameMargin: int,
             chunks.append([startP, j, int(includeFrame[j-1])])
             startP = j
     chunks.append([startP, arrayLen, int(includeFrame[j])])
-
-    # Return both formatted and long numpy array.
-    return chunks, includeFrame
+    return chunks
