@@ -1,14 +1,13 @@
 '''fastAudio.py'''
 
 from usefulFunctions import ProgressBar, getNewLength
-from mediaMetadata import getAudioCodec
 
 def convertAudio(ffmpeg, ffprobe, theFile, INPUT_FILE, outFile, args, log):
     log.debug(f'Convering internal audio file: {theFile} to {outFile}')
 
     realCodec = args.audio_codec
     if(realCodec is None):
-        realCodec = getAudioCodec(ffprobe, INPUT_FILE)
+        realCodec = ffprobe.getAudioCodec(INPUT_FILE)
     if(realCodec == "pcm_s16le" and outFile.endswith('.m4a')):
         log.error(f'Codec: {realCodec} is not supported in the m4a container.')
 
@@ -34,8 +33,10 @@ def fastAudio(theFile, outFile, chunks: list, speeds: list, log, fps: float,
     import os
 
     import numpy as np
-    from audiotsm2 import phasevocoder
-    from audiotsm2.io.array import ArrReader, ArrWriter
+
+    if((speeds[0] != 1 and speeds[0] != 99999) or (speeds[1] != 1 and speeds[1] != 99999)):
+        from audiotsm2 import phasevocoder
+        from audiotsm2.io.array import ArrReader, ArrWriter
 
     log.checkType(chunks, 'chunks', list)
     log.checkType(speeds, 'speeds', list)
