@@ -31,19 +31,19 @@ class Log():
 
         from shutil import rmtree
         rmtree(self.temp)
-
-        if(self.is_debug):
-            self.debug(f'   - Removed Temp Directory.')
+        self.debug(f'   - Removed Temp Directory.')
 
     def error(self, message):
         print('Error!', message, file=sys.stderr)
         self.cleanup()
         sys.exit(1)
 
-    # When something's definitely wrong with the program.
     def bug(self, message, bug_type='bug report'):
-        print('Error!', message, f'\n\nCreate a {bug_type} at',
-            'https://github.com/WyattBlue/auto-editor/issues/\n', file=sys.stderr)
+        URL = 'https://github.com/WyattBlue/auto-editor/issues/'
+        print('Error!', message,
+            "\n\nThis is not a normal error.\n This message will only show up if there's",
+            'something definitely wrong with the program.',
+            f'\nCreate a {bug_type} at:\n  {URL}', file=sys.stderr)
         self.cleanup()
         sys.exit(1)
 
@@ -91,9 +91,11 @@ def pipeToConsole(myCommands: list) -> str:
 
 
 class FFprobe():
-    def __init__(self, plat, dirPath, myFFmpeg: bool):
+    def __init__(self, plat, dirPath, myFFmpeg: bool, log):
         from os import path
 
+        self.mylog = log
+
         newF = None
         if(plat == 'Windows' and not myFFmpeg):
             newF = path.join(dirPath, 'win-ffmpeg/bin/ffprobe.exe')
@@ -104,23 +106,107 @@ class FFprobe():
             self.myPath = newF
         else:
             self.myPath = 'ffprobe'
+            try:
+                pipeToConsole([self.myPath, '-h'])
+            except FileNotFoundError:
+                if(plat == 'Darwin'):
+                    self.log.error('No ffprobe found, download via homebrew or restore' \
+                        ' the included binary.')
+                elif(plat == 'Windows'):
+                    self.log.error('No ffprobe found, download ffprobe with your' \
+                        ' favorite package manager (ex chocolatey), or restore the' \
+                        ' included binary.')
+                else:
+                    self.log.error('ffprobe must be on PATH. Download ffprobe by running:\n' \
+                        '  sudo apt-get install libavformat-dev libavfilter-dev libavdevice-dev ffmpeg' \
+                        '\nOr something similar depending on your distro.')
+
+    def log(self, message):
+        self.mylog.ffmpeg(message)
 
     def getPath(self) -> str:
         return self.myPath
 
+    def updateLog(self, log):
+        self.mylog = log
+
     def run(self, cmd: list):
         import subprocess
-        subprocess.call([self.myPath] + cmd)
+        full_cmd = [self.myPath] + cmd
+
+        self.log(full_cmd)
+        subprocess.call(full_cmd)
 
     def pipe(self, cmd: list) -> str:
-        return pipeToConsole([self.myPath, '-v', 'error'] + cmd)
+        full_cmd = [self.myPath, '-v', 'error'] + cmd
 
+        self.log(full_cmd)
+        output = pipeToConsole(full_cmd)
+        self.log(output)
+
+        return output
+
+    def _get(self, file, stream, the_type, track, of='compact=p=0:nk=1') -> str:
+        return self.pipe(['-select_streams', f'{the_type}:{track}', '-show_entries',
+            f'stream={stream}', '-of', of, file]).strip()
+
+    def getResolution(self, file):
+        return self._get(file, 'height,width', 'v', 0, of='csv=s=x:p=0')
+
+    def getDuration(self, file):
+        return self._get(file, 'duration', 'v', 0)
+
+    def getFrameRate(self, file):
+        output = self.pipe(['-select_streams', 'v', '-show_entries',
+            'stream=avg_frame_rate', '-of', 'compact=p=0:nk=1', file]).strip()
+        nums = output.split('/')
+        return int(nums[0]) / int(nums[1])
+
+    def getAudioTracks(self, file):
+        output = self.pipe(['-select_streams', 'a', '-show_entries', 'stream=index',
+            '-of', 'compact=p=0:nk=1', file]).strip()
+
+        numbers = output.split('\n')
+        # Remove all \r chars that can appear in certain environments
+        numbers = [s.replace('\r', '') for s in numbers]
+        # Remove all blanks
+        numbers = [s for s in numbers if s != '']
+
+        self.log('Track data: ' + str(numbers))
+        if(numbers[0].isnumeric()):
+            return len(numbers)
+        else:
+            self.mylog.warning('ffprobe had an invalid output.')
+            return 1 # Assume there's one audio track.
+
+    def getAudioCodec(self, file, track=0):
+        return self._get(file, 'codec_name', 'a', track)
+
+    def getSampleRate(self, file, track=0):
+        return self._get(file, 'sample_rate', 'a', track)
+
+    def getAudioBitrate(self, file, track=0):
+        return self._get(file, 'bit_rate', 'a', track)
+
+    def getPrettySampleRate(self, file, track=0) -> str:
+        output = self.getSampleRate(file, track)
+        if(output.isnumeric()):
+            return str(int(output) / 1000) + ' kHz'
+        return 'N/A'
+
+    def getPrettyABitrate(self, file, track=0) -> str:
+        output = self.getAudioBitrate(file, track)
+        if(output.isnumeric()):
+            # This does get used by ffmpeg so be careful.
+            return str(round(int(output) / 1000)) + 'k'
+        return 'N/A'
 
 class FFmpeg():
-    def __init__(self, plat, dirPath, myFFmpeg: bool, show: bool):
+    def __init__(self, plat, dirPath, myFFmpeg: bool, log):
         from os import path
 
-        self.show = show
+        self.mylog = log
+
         newF = None
         if(plat == 'Windows' and not myFFmpeg):
             newF = path.join(dirPath, 'win-ffmpeg/bin/ffmpeg.exe')
@@ -131,18 +217,37 @@ class FFmpeg():
             self.myPath = newF
         else:
             self.myPath = 'ffmpeg'
+            try:
+                pipeToConsole([self.myPath, '-h'])
+            except FileNotFoundError:
+                if(plat == 'Darwin'):
+                    self.log.error('No ffmpeg found, download via homebrew or restore' \
+                        ' the included binaries.')
+                elif(plat == 'Windows'):
+                    self.log.error('No ffmpeg found, download ffmpeg with your' \
+                        ' favorite package manager (ex chocolatey), or restore the' \
+                        ' included binaries.')
+                else:
+                    self.log.error('FFmpeg must be on PATH. Download ffmpeg by running:\n' \
+                        '  sudo apt-get install libavformat-dev libavfilter-dev libavdevice-dev ffmpeg' \
+                        '\nOr something similar depending on your distro.')
+
+    def log(self, message):
+        self.mylog.ffmpeg(message)
 
     def getPath(self) -> str:
         return self.myPath
 
+    def updateLog(self, log):
+        self.mylog = log
+
     def run(self, cmd: list):
         cmd = [self.myPath, '-y'] + cmd
-        if(self.show):
+        if(self.mylog.is_ffmpeg):
             cmd.extend(['-hide_banner'])
         else:
             cmd.extend(['-nostats', '-loglevel', '8'])
-        if(self.show):
-            print(cmd)
+        self.log(cmd)
 
         import subprocess
         subprocess.call(cmd)
@@ -150,11 +255,9 @@ class FFmpeg():
     def pipe(self, cmd: list) -> str:
         cmd = [self.myPath, '-y'] + cmd
 
-        if(self.show):
-            print(cmd)
+        self.log(cmd)
         output = pipeToConsole(cmd)
-        if(self.show):
-            print(output)
+        self.log(output)
 
         return output
 
@@ -168,19 +271,20 @@ def getNewLength(chunks: list, speeds: list, fps: float) -> float:
     return timeInFrames / fps
 
 
-def prettyTime(myTime: float) -> str:
+def prettyTime(myTime: float, ampm: bool) -> str:
     newTime = localtime(myTime)
 
     hours = newTime.tm_hour
     minutes = newTime.tm_min
 
-    if(hours == 0):
-        hours = 12
-    if(hours > 12):
-        hours -= 12
-    ampm = 'PM' if newTime.tm_hour >= 12 else 'AM'
-    return f'{hours:02}:{minutes:02} {ampm}'
-
+    if(ampm):
+        if(hours == 0):
+            hours = 12
+        if(hours > 12):
+            hours -= 12
+        ampm = 'PM' if newTime.tm_hour >= 12 else 'AM'
+        return f'{hours:02}:{minutes:02} {ampm}'
+    return f'{hours:02}:{minutes:02}'
 
 def bar(termsize, title, doneStr, togoStr, percentDone, newTime):
     bar = f'  ⏳{title}: [{doneStr}{togoStr}] {percentDone}% done ETA {newTime}'
@@ -200,8 +304,19 @@ class ProgressBar():
         self.len_title = len(title)
         self.machine = machineReadable
         self.hide = hide
+        self.ampm = True
 
-        newTime = prettyTime(self.beginTime)
+        if(not self.machine):
+            import platform
+            if(platform.system() == 'Darwin'):
+                try:
+                    dateFormat = pipeToConsole(['defaults', 'read',
+                        'com.apple.menuextra.clock', 'DateFormat'])
+                    self.ampm = 'a' in dateFormat
+                except FileNotFoundError:
+                    pass
+
+        newTime = prettyTime(self.beginTime, self.ampm)
         termsize = get_terminal_size().columns
 
         if(hide):
@@ -231,7 +346,7 @@ class ProgressBar():
         else:
             percentPerSec = (time() - self.beginTime) / percentDone
 
-        newTime = prettyTime(self.beginTime + (percentPerSec * 100))
+        newTime = prettyTime(self.beginTime + (percentPerSec * 100), self.ampm)
 
         if(self.machine):
             index = min(index, self.total)
@@ -277,7 +392,6 @@ def humanReadableTime(rawTime: float) -> str:
         units = 'minutes'
     return f'{rawTime} {units}'
 
-
 def smartOpen(newOutput: str, log):
     from subprocess import call
     try:  # should work on Windows
